/*
** File: timer.c  
** Project: VxWorks Driver Framework
** Purpose: Timer functions
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"

void
_dfVxwTimerStub(
#if DFVXW_HAS_VX_USR_ARG_T
  _Vx_usr_arg_t arg)
#else
  int arg)
#endif
{
  DfTimer* pDfTimer = (DfTimer*)arg;
  STATUS result;
  DfTime now, delayRemaining;
  int delay;

  dfDebugPrint(5, ("_dfVxwTimerStub: entered, pDfTimer=%p\n", pDfTimer));
  if (pDfTimer->system.bFinalDelay) {
    dfDebugPrint(5, ("_dfVxwTimerStub: calling callback\n"));
    pDfTimer->system.pCallback(pDfTimer, pDfTimer->system.pTimerContext, pDfTimer->system.pCallbackContext);
  } else {
    dfDebugPrint(5, ("_dfVxwTimerStub: rescheduling\n"));
    now = dfTimeGet();
    delayRemaining = pDfTimer->system.expires - now;
    if (delayRemaining > INT_MAX) {
      delay = INT_MAX;
    } else {
      pDfTimer->system.bFinalDelay = TRUE;
      delay = (int)delayRemaining;
    }
#if DFVXW_HAS_VX_USR_ARG_T
    result = wdStart(pDfTimer->system.wdogId, delay, (FUNCPTR)_dfVxwTimerStub, (_Vx_usr_arg_t)pDfTimer);
#else
    result = wdStart(pDfTimer->system.wdogId, delay, (FUNCPTR)_dfVxwTimerStub, (int)pDfTimer);
#endif
    dfAssert(OK == result);
  }
}

void
dfTimerInit(
  DfTimer* pDfTimer,
  void* pTimerContext)
{
  pDfTimer->system.wdogId = wdCreate();
  dfAssert(NULL != pDfTimer->system.wdogId);
  pDfTimer->system.pTimerContext = pTimerContext;
}

void
dfTimerUninit(
  DfTimer* pDfTimer)
{
  wdDelete(pDfTimer->system.wdogId);
}
